/* Program finds an approximate solution of the equation f(x) = 0
   in a given interval, using the bisection method.
   Input:   Desired accuracy of approximation, endpoints of an
            interval containing a solution
   Output:  Prompts to the user and the approximate solution
------------------------------------------------------------------*/

#include <iostream>
using namespace std;

double f(double x);

int main()
{
  cout << "\nThis program uses the bisection method to find an\n"
       << "approximate solution to the equation f(x) = 0.\n";

  double desiredAccuracy;         // the accuracy desired

  cout << "\nEnter the accuracy desired (e.g. .001): ";
  cin >> desiredAccuracy;

  double left, right;             // get interval containing a solution

  do
  {
    cout << "Enter endpoints of an interval containing a solution: ";
    cin >> left >> right;
  }
  while (f(left) * f(right) >= 0.0);

  double width = right - left,    // the interval width
         midPt,                   // the midpoint of the interval
         fMid;                    // value of f() at midPoint

  while (width/2.0 > desiredAccuracy)
  {
    midPt = (left + right) / 2.0;
    fMid = f(midPt);

    if (f(left) * fMid < 0.0 )    //   solution is in left half
      right = midPt;
    else                          //   solution is in right half
       left = midPt;

    width /= 2.0;               //   split the interval
  }


  cout << "\n-->" << midPt << " is an approximate solution of f(x) = 0,"
       << " to within " << desiredAccuracy << endl;
}

// Function for which a root is being found
double f(double x)
{
   return x*x*x + x - 5;
}
